# -*- coding: utf-8 -*-
#
#       storyimportthread.py
#       
#       Copyright © 2009, 2010 Per Liedman <per@liedman.net>
#       Copyright © 2011 Brandon Invergo <b.invergo@gmail.com>
#       
#       This file is part of Grotesque.
#
#       Grotesque is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Grotesque is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Grotesque.  If not, see <http://www.gnu.org/licenses/>.


import threading
import os

from gi.repository import Gtk, Gdk

from treatyofbabel import babel, ifdb, storydata


class StoryImportThread():
    '''This class handles importing stories into the library in a threaded
    manner. This allows a progress bar to be displayed since the process can
    take some time.
    
    '''
    def __init__(self, filenames, settings, library, dialog):
        self.settings = settings
        self.filenames = filenames
        self.coverart_dir = settings.coverart_dir
        self.fetch_metadata = settings.get_fetch_metadata()
        self.fetch_coverart = settings.get_fetch_coverart()
        self.library = library
        self.dialog = dialog
        self.stopthread = False
        self.fails = []

    def stop(self):
        self.stopthread = True

    def import_stories(self):
        self.msg = ''
        count = 0
        for filename in self.filenames:
            if self.stopthread:
                yield False
            # Ignore files that are already in the self.library.
            if self.library.story_in_library(filename):
                continue
            story = None
            xmlfile = None
            basename = os.path.split(filename)[1]
            yield True
            self.dialog.info_label.set_text('Importing...')
            self.dialog.progressbar.set_text(basename)
            # Attempt to extract the metadata from the file.
            try:
                story = babel.get_storydata_from_filename(filename,
                                                          self.coverart_dir)
            except:
                story = storydata.StoryData()
                story.title = basename
                story.story_file = filename
                ext = os.path.splitext(filename)[1].strip('.')
                story.format = self.settings.get_format_from_ext(ext)
                story.get_local_coverart(self.coverart_dir)
                self.fails.append(story)
                continue
            yield True
            # If it completely failed, add it to the error message.
            if not story:
                self.msg = ''.join([self.msg,
                                   'Failed to import "{0}".\n'.format(
                                   os.path.split(filename)[1])])
            # If it failed to extract an IFID, we can't do anything more with
            # it, so add it to a list of failed extractions whose metadata
            # the user can edit.
            #elif story.is_empty(): #story.ifid is None:
            #    self.fails.append(story)
            else:
                # We have the IFID, but we don't have a title, so we should
                # fetch metadata from IFDB.
                if story.is_empty(): #story.title == None or len(story.title) == 0:
                    # Only fetch if the user has specified to do so in the 
                    # self.settings.
                    if self.fetch_metadata:
                        progress_text = \
                            "{0}: Getting metadata from IFDB".format(basename)
                        self.dialog.progressbar.set_text(progress_text)
                        # Fill the story object with metadata from IFDB. Keep
                        # the XML file so we don't have to request it from the
                        # server again if we're going to download the cover
                        # art.
                        xmlfile = ifdb.fill_meta_from_ifdb(story)
                        yield True
                    # If, after IFDB, we still don't have metadata, add the 
                    # file to the list of failures.
                    if story.is_empty(): #story.title == None or len(story.title) == 0:
                        story.title = os.path.splitext(basename)[0]
                        self.fails.append(story)
                    # Otherwise, add the story to the self.library.
                    else:
                        self.library.add_entry(story)
                else:
                    # Or, if we successfully extracted metadata from the file,
                    # be sure it's in the right encoding.
                    #for item in [story.title, story.author, story.language,
                                 #story.headline, story.firstpublished,
                                 #story.genre, story.group, story.series,
                                 #story.seriesnumber, story.forgiveness,
                                 #story.description]:
                        #if item:
                            #item.encode('UTF-8')
                    # Fetch the star rating from IFDB, which won't be embedded
                    # in the file.
                    if self.fetch_metadata:
                        ifdb.fetch_rating(story)
                        yield True
                    # And add the story to the self.library.
                    self.library.add_entry(story)
                if self.fetch_coverart:
                    # If the user wants coverart fetched, do it here.
                    progress_text = \
                            "{0}: Getting cover art from IFDB".format(basename)
                    self.dialog.progressbar.set_text(progress_text)
                    ifdb.fetch_cover_art(story, self.coverart_dir, xmlfile)
            count = count + 1
            # Update the progress bar.
            self.dialog.progressbar.set_fraction(float(count) /
                                                 float(len(self.filenames)))
            yield True
        self.dialog.response(Gtk.ResponseType.OK)
        yield False
