# -*- coding: utf-8 -*-
#
#       ifiction.py
#       
#       Copyright © 2009, 2010 Per Liedman <per@liedman.net>
#       Copyright © 2011 Brandon Invergo <b.invergo@gmail.com>
#       
#       This file is part of Grotesque.
#
#       Grotesque is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Grotesque is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Grotesque.  If not, see <http://www.gnu.org/licenses/>.


import xml.dom.minidom
import re

from storydata import StoryData

identification_attributes = ['ifid', 'format']
bibliographic_attributes = ['title', 'author', 'language', 'headline',
                            'firstpublished', 'genre', 'group', 'description',
                            'series', 'seriesnumber', 'forgiveness']


class IFictionError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


def get_stories_from_ifiction(ifiction):
    dom = xml.dom.minidom.parseString(ifiction)
    root = dom.documentElement
    if root.localName == 'ifindex':
        return [handle_story(StoryData(), story) \
                    for story in root.childNodes \
                    if (story.localName is not None and
                    story.localName.startswith('story'))]
    elif root.localName == 'story':
        return [handle_story(StoryData(), root)]
    else:
        raise IFFictionError('Unknown XML root tag "' + root.localName + '".')


def get_text(nodelist):
    rc = ""
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc = rc + node.data
    return rc


def get_text_from_child(node, name):
    nodes = node.getElementsByTagName(name)
    if len(nodes) > 0:
        return get_text(nodes[0].childNodes).encode('UTF-8')
    else:
        return None


def handle_story(s, story):
    identification = story.getElementsByTagName('identification')
    bibliographic = story.getElementsByTagName('bibliographic')
    zoom = story.getElementsByTagName('zoom')
    if len(identification) > 0:
        handle_identification(s, identification[0])
    else:
        raise IFFictionError('ifiction is missing the identification tag')
    if len(bibliographic) > 0:
        handle_bibliographic(s, bibliographic[0])
    if len(zoom) > 0:
        handle_zoom(s, zoom[0])
    if s.format in ['tads2', 'tads3'] and s.author is None:
        tads = story.getElementsByTagName('tads')
        handle_tads(s, tads[0])
    return s


def handle_identification(s, id_tag):
    for attrib in identification_attributes:
        setattr(s, attrib, get_text_from_child(id_tag, attrib))


def handle_bibliographic(s, bib):
    for attrib in bibliographic_attributes:
        setattr(s, attrib, get_text_from_child(bib, attrib))

def handle_tads(s, bib):
    byline = get_text_from_child(bib, 'byline')
    if 'by' in byline or 'By' in byline:
        author = re.match('[Bb]y (.+)', byline).group(1)
    else:
        author = byline
    s.author = author

def handle_zoom(s, zoom):
    setattr(s, 'story_file', get_text_from_child(zoom, 'story'))


if __name__ == "__main__":
    import sys
    with open(sys.argv[1]) as f:
        ifiction = f.read()
        stories = get_stories_from_ifiction(ifiction)
        print stories
        for s in stories:
            print s.title, 'a', s.genre, 'story by', s.author, " - IFID:", \
                s.ifid, " - format:", s.format
