# -*- coding: utf-8 -*-
#
#       tads.py
#       
#       Copyright © 2009, 2010 Per Liedman <per@liedman.net>
#       Copyright © 2011 Brandon Invergo <b.invergo@gmail.com>
#       
#       This file is part of Grotesque.
#
#       Grotesque is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Grotesque is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Grotesque.  If not, see <http://www.gnu.org/licenses/>.


import struct
import re
import md5
import os.path
from binascii import hexlify

from storydata import StoryData
from ifformaterror import IFFormatError
from _binaryfuncs import read_int, read_short, read_byte, read_long

T2_SIGNATURE = 'TADS2 bin\012\015\032'
T3_SIGNATURE = 'T3-image\015\012\032'
HTML_RES_ID = 'HTMLRES'
EOF_RES_ID = '$EOF'


attribs = [('name', 'title'), ('title', 'title'), ('ifid', 'ifid'),
           ('byline', 'author', re.compile('[Bb]y (.+)')),
           ('authoremail', 'author'), ('headline', 'headline'),
           ('desc', 'description'), ('firstpublished', 'firstpublished'),
           ('language', 'language'), ('series', 'series'), 
           ('seriesnumber', 'seriesnumber'), ('genre', 'genre'),
           ('forgiveness', 'foregivness')]


def get_extensions():
    return ['gam', 't3']


def get_information(file_buffer):
    s = StoryData()
    if is_tads2(file_buffer):
        s.format = 'tads2'
    elif is_tads3(file_buffer):
        s.format = 'tads3'
    else:
        raise IFFormatError('file_buffer is neither TADS2 or TADS3.')
    gameinfo = get_gameinfo(file_buffer)
    if gameinfo is not None:
        for attrib in attribs:
            value = gameinfo.get(attrib[0])
            if value is not None:
                if len(attrib) > 2:
                    match = attrib[2].match(value)
                    if match is not None:
                        value = match.group(1)
                elif  attrib[0] == 'authoremail':
                    value = parse_authors(value)
                setattr(s, attrib[1], value)
    if s.ifid is None:
        file_hash = hexlify(md5.new(file_buffer).digest()).upper()
        if is_tads2(file_buffer):
            s.ifid = ''.join(['TADS2-', file_hash])
        elif is_tads3(file_buffer):
            s.ifid = ''.join(['TADS3-', file_hash])
        else:
            raise IFFormatError('file_buffer is neither TADS2 or TADS3.')
    #cover = get_cover(file_buffer)
    #if cover is not None:
        #s.coverart = cover
    return s
    
    
def is_a(file_buffer):
    return is_tads2(file_buffer) or is_tads3(file_buffer)


def get_gameinfo(file_buffer):
    rsc = find_resource(file_buffer, 'GameInfo.txt')
    if rsc is not None:
        #rsc = rsc.decode('utf-8')
        gameinfo = {}
        for line in rsc.split('\n'):
            line = line.strip()
            if not line.startswith('#'):
                parts = line.split(':', 1)
                if len(parts) == 2:
                    key = parts[0].strip().lower()
                    value = parts[1].strip().replace('\\n', '\n')
                    value = cleanup_info(value)
                    gameinfo[key] = value
        return gameinfo
    else:
        return None

def cleanup_info(value):
    cleaned = re.sub('\s{2,}', ' ', value)
    return cleaned


def find_resource(file_buffer, name):
    if is_tads2(file_buffer):
        return t2_find_resource(file_buffer, name)
    elif is_tads3(file_buffer):
        return t3_find_resource(file_buffer, name)
    else:
        raise IFFormatError('file_buffer is neither TADS2 or TADS3.')


def t2_find_resource(file_buffer, name):
    name = name.lower()

    # Skip past TADS2 header (13 bytes signature, 7 bytes version, 2 bytes
    # flags, 26 bytes timestamp
    p = 13 + 7 + 2 + 26
    while p < len(file_buffer):
        type_length = read_byte(file_buffer, p, '<')
        next_section = read_int(file_buffer, p + 1 + type_length, '<')
        type = file_buffer[p + 1:p + 1 + type_length]
        if type == HTML_RES_ID:
            # Skip past the section block header
            p = p + 12
            number_entries = read_int(file_buffer, p, '<')
            # Skip the first index entry
            p = p + 8
            # Scan index entries
            found_offset = None
            found_length = None
            for x in range(0, number_entries):
                rsc_offset = read_int(file_buffer, p, '<')
                rsc_size = read_int(file_buffer, p + 4, '<')
                name_len = read_short(file_buffer, p + 8, '<')
                p = p + 10
                rsc_name = file_buffer[p:p + name_len]
                if name_len == len(name) and rsc_name.lower() == name:
                    found_offset = rsc_offset
                    found_length = rsc_size
                p = p + name_len
            if found_offset is not None:
                return file_buffer[p + found_offset:p + found_offset +
                                   found_length]
        elif type == EOF_RES_ID:
            return None
        p = next_section
    return None


def t3_find_resource(file_buffer, name):
    name = name.lower()
    rsc_name_len = len(name)

    # Skip past TADS3 header (11 bytes signature, 2 bytes version, 32 bytes
    # reserved, 24 bytes timestamp
    p = 11 + 2 + 32 + 24
    while p < len(file_buffer):
        rsc_type = file_buffer[p:p + 4]
        block_size = read_int(file_buffer, p + 4, '<')
        if rsc_type == 'MRES':
            # Skip past the section block header
            p = p + 10
            block_start = p
            number_entries = read_short(file_buffer, p, '<')
            p = p + 2
            # Scan index entries
            for x in range(number_entries):
                rsc_offset = read_int(file_buffer, p, '<')
                rsc_size = read_int(file_buffer, p + 4, '<')
                name_len = read_byte(file_buffer, p + 8, '<')
                rsc_name = ''
                for xored_char in file_buffer[p + 9:p + 9 + rsc_name_len]:
                    rsc_name = rsc_name + chr(ord(xored_char) ^ 0xFF)
                if name_len == rsc_name_len and rsc_name.lower() == name:
                    return file_buffer[block_start + rsc_offset:\
                                      block_start + rsc_offset + rsc_size]
                p = p + 9 + name_len
            p = block_start + block_size
        elif rsc_type == 'EOF':
            return None
        else:
            p = p + 10 + block_size
    return None


def is_tads2(file_buffer):
    return file_buffer.startswith(T2_SIGNATURE)


def is_tads3(file_buffer):
    return file_buffer.startswith(T3_SIGNATURE)


def parse_authors(author_str):
    author_list = []
    author_split = author_str.split(';')
    for author in author_split:
        author_list.append(author.split('<')[0].strip())
    if len(author_list) == 1:
        authors = author_list[0]
    elif len(author_list) == 2:
        authors = ' and '.join(authors)
    elif len(author_list) > 2:
        authors = ', '.join([', '.join(author_list[:-2]),
                             ' and '.join(author_list[-2:])])
    return authors


def matches(file_buffer, offset, string):
    return file_buffer[offset:offset + len(string)] == string


def get_cover_extent(file_buffer):
    ret, res, typ, width, height = find_cover_art(file_buffer)
    return len(res)
        
    
def get_cover_format(file_buffer):
    ret, res, typ, width, height = find_cover_art(file_buffer)
    return typ


def get_cover(file_buffer):
    res, typ, width, height = find_cover_art(file_buffer)
    return (typ, res)


def find_cover_art(file_buffer):
    for resc_name in ['CoverArt.jpg', 'CoverArt.png']:
        cover_resc = find_resource(file_buffer, resc_name)
        if cover_resc:
            ext = os.path.splitext(resc_name)[1].strip('.')
            #if ext == 'jpg':
                #dims = get_jpeg_dim(cover_resc)
            #else: 
                #dims = get_png_dim(cover_resc)
            #print dims
            #if not dims[0] and not dims[1]:
                #return (None, None, None, None)
            width = None #dims[0]
            height = None #dims[1]
            return (cover_resc, ext, width, height)
    return (None, None, None, None)


def get_jpeg_dim(img):
    if read_int(img, 0) != 0xff or read_int(img, 1) != 0xD8:
        return (None, None)
    dp = 2
    ep = len(img)
    while True:
        if dp > ep:
            return (None, None)
        t1 = read_int(img, dp)
        dp += 1
        while t1 != 0xff:
            if dp > ep:
                return (None, None)
            t1 = read_int(img, dp)
            dp += 1
        t1 = read_int(img, dp)
        dp += 1
        while t1 == 0xff:
            if dp > ep:
                return (None, None)
            t1 = read_int(img, dp)
            dp += 1
        if t1 & 0xF0 == 0xC0 and not (t1 == 0xC4 or t1 == 0xC8 or t1 == 0xCC):
            dp += 3
            if dp > ep:
                return (None, None)
            h = read_int(img, dp) * 2**8
            dp += 1
            if dp > ep:
                return (None, None)
            h = h | read_int(img, dp)
            dp += 1
            if dp > ep:
                return (None, None)
            w = read_int(img, dp) * 2**8
            dp += 1
            if dp > ep:
                return (None, None)
            w = w | read_int(img, dp)
            return (w, h)
        elif t1 == 0xD8 or t1 == 0xD9:
            break
        else:
            if dp > ep:
                return (None, None)
            l = read_int(img, dp) * 2**8
            dp += 1
            if dp > ep:
                return (None, None)
            l = l | read_int(img, dp)
            l -= 2
            dp += l
            if dp > ep:
                return (None, None)
    return (None, None)


def get_png_dim(img):
    if (len(img) < 33 or not (ord(img[0]) == 137 and ord(img[1]) == 80 and
                              ord(img[2]) == 78 and ord(img[3]) == 71 and
                              ord(img[4]) == 13 and ord(img[5]) == 10 and 
                              ord(img[6]) == 26 and ord(img[7]) == 10) or
                         not (img[12] == 'I' and img[13] == 'H' and
                              img[14] == 'D' and img[15] == 'R')):
        return (None, None)
    w = read_long(img, 16)
    h = read_long(img, 20)
    return (w, h)


if __name__ == "__main__":
    import sys
    name = sys.argv[1]
    with open(name) as f:
        data = f.read()
        s = get_information(data)
        print s.title
        print s.author
        print s.description
        print s.ifid
        print s.coverart
